import { app } from '@/app.ts';
import { AiChatHistoryModel } from './models/ai-chat-history.ts';

app
  .route({
    path: 'ai',
    key: 'get-chat-list',
    description: '获取chat列表',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const type = ctx.query.type || 'keep';
    const aiChatList = await AiChatHistoryModel.findAll({
      where: {
        uid: tokenUser.id,
        type,
      },
      order: [['updatedAt', 'DESC']],
    });
    ctx.body = {
      list: aiChatList,
    };
  })
  .addTo(app);

app
  .route({
    path: 'ai',
    key: 'update-chat',
    description: '更新chat',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const uid = tokenUser.id;
    const { id, data, prompt_tokens, total_tokens, completion_tokens, createdAt, updatedAt, ...rest } = ctx.query.data || {};
    let aiChat: AiChatHistoryModel | null = null;
    if (id) {
      aiChat = await AiChatHistoryModel.findByPk(id);
      if (!aiChat) {
        ctx.throw(404, 'chat not found');
      }
      if (aiChat.uid !== uid) {
        ctx.throw(403, 'no permission');
      }
      await aiChat.update({ data: { ...aiChat.data, ...data }, ...rest, version: aiChat.version + 1 });
    } else {
      aiChat = await AiChatHistoryModel.create({
        ...rest,
        uid: uid,
      });
    }
    ctx.body = aiChat;
  })
  .addTo(app);

app
  .route({
    path: 'ai',
    key: 'get-chat',
    description: '获取chat',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { id } = ctx.query.data || {};
    if (!id) {
      ctx.throw(400, 'id is required');
    }
    const aiChat = await AiChatHistoryModel.findByPk(id);
    if (!aiChat) {
      ctx.throw(404, 'chat not found');
    }
    if (aiChat.uid !== tokenUser.id) {
      ctx.throw(403, 'no permission');
    }
    ctx.body = aiChat;
  })
  .addTo(app);
app
  .route({
    path: 'ai',
    key: 'get-chat-version',
    description: '获取chat版本',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { id } = ctx.query.data || {};
    if (!id) {
      ctx.throw(400, 'id is required');
    }
    const aiChat = await AiChatHistoryModel.findByPk(id);
    if (!aiChat) {
      ctx.throw(404, 'chat not found');
    }
    if (aiChat.uid !== tokenUser.id) {
      ctx.throw(403, 'no permission');
    }
    ctx.body = {
      id: aiChat.id,
      version: aiChat.version,
    };
  })
  .addTo(app);

app
  .route({
    path: 'ai',
    key: 'delete-chat',
    description: '删除chat',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { id } = ctx.query.data || {};
    if (!id) {
      ctx.throw(400, 'id is required');
    }
    const aiChat = await AiChatHistoryModel.findByPk(id);
    if (!aiChat) {
      ctx.throw(404, 'chat not found');
    }
    if (aiChat.uid !== tokenUser.id) {
      ctx.throw(403, 'no permission');
    }
    await aiChat.destroy();

    ctx.body = {
      success: true,
    };
  })
  .addTo(app);
